import .imp_syntax
import .imp_common
import .imp_eval

namespace imp

open stmt
open val

inductive step :
  store → stmt →
  store → stmt → Prop
| set :
    forall s x e v,
      eval_e s e v →
      step
        s (set x e)
        (update x v s) nop
| ifelse_t :
    forall s e p1 p2,
      eval_e s e (bool tt) →
      step
        s (ifelse e p1 p2)
        s p1
| ifelse_f :
    forall s e p1 p2,
      eval_e s e (bool ff) →
      step
        s (ifelse e p1 p2)
        s p2
| while_t :
    forall s e p,
      eval_e s e (bool tt) →
      step
        s (while e p)
        s (seq p (while e p))
| while_f :
    forall s e p,
      eval_e s e (bool ff) →
      step
        s (while e p)
        s nop
| seq_nop :
    forall s p2,
      step
        s (seq nop p2)
        s p2
| seq :
    forall s p1 p2 s' p1',
      step
        s p1
        s' p1' →
      step
        s (seq p1 p2)
        s' (seq p1' p2)


inductive step_star :
  store → stmt →
  store → stmt → Prop
| refl :
    forall s p,
      step_star
        s p
        s p
| left :
    forall s1 p1 s2 p2 s3 p3,
      step
        s1 p1
        s2 p2 →
      step_star
        s2 p2
        s3 p3 →
      step_star
        s1 p1
        s3 p3

end imp
